/* ----------------------------------------------------------------------
 * FILE: main.c
 * PACKAGE: as31 - 8031/8051 Assembler.
 *
 * DESCRIPTION:
 *	The file contains main(). It handles the arguments and makes
 *	sure that pass 1 is done before pass 2 etc...
 *
 * REVISION HISTORY:
 *	Jan. 19, 1990 - Created. (Ken Stauffer)
 *
 * AUTHOR:
 *	All code in this file written by Ken Stauffer (University of Calgary).
 *	January, 1990. "Written by: Ken Stauffer"
 *
 *	April, 2000, Paul Stoffregen: see Makefile for details
 */

#include <stdio.h>
#include <setjmp.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include "as31.h"


void quit(int exit_code);


/* global variables */
unsigned long lc;
char *asmfile;
int fatal=0, abort_asap=0;
int pass=0;
int dashl=0;
FILE *listing=NULL, *fin=NULL;


int main(int argc, char **argv)
{
	char *dashF=NULL, *dashA=NULL, *outfile=NULL, *lstfile=NULL;
	const char *extension;
	int use_stdout=0, has_dot_asm=0;
	int i, len, baselen, extlen;

	fprintf(stderr, "AS31, beta test release, April 20, 2000\n");
	fprintf(stderr, "Please report problems to: paul@pjrc.com\n\n");


	if (argc < 2) {
		fprintf(stderr,
			"Usage: %s [-l] [-s] [-Ffmt] [-Aarg] file.asm\n",
			argv[0]);
		fprintf(stderr, "\t -l :  create list file\n");
		fprintf(stderr, "\t -s :  send output to stdout\n");
		fprintf(stderr, "\t -F :  output format (intel hex default)\n");
		fprintf(stderr, "\t -A :  optional output format argument\n");
		emitusage();
		exit(1);
	}

	for (i=1; i<argc; i++ ) {
		if( argv[i][0] != '-' ) break;
		if( argv[i][1] == 'l' )
			dashl = 1;
		else if( argv[i][1] == 's' )
			use_stdout = 1;
		else if( dashF == NULL && argv[i][1] == 'F' )
			dashF = argv[i]+2;
		else if( dashA == NULL && argv[i][1] == 'A' )
			dashA = argv[i]+2;
		else {
			fprintf(stderr,"Duplicate or unknown flag.\n");
			exit(1);
		}
	}
	if (i == argc) {
		fprintf(stderr,"Missing input file.\n");
		exit(1);
	}

	extension = emit_extension(dashF);
	extlen = strlen(extension);

	len = baselen = strlen(argv[i]);
	if (len >= 4 && strcasecmp(argv[i] + len - 4, ".asm") == 0) {
		has_dot_asm = 1;
		baselen -= 4;
	}
	
	asmfile = (char *)malloc(baselen + 5);
	strcpy(asmfile, argv[i]);

	if (dashl) {
		lstfile = (char *)malloc(baselen + 5);
		strncpy(lstfile, argv[i], baselen);
		strcpy(lstfile + baselen, ".lst");
	}

	if (use_stdout) {
		outfile = NULL;
	} else {
		outfile = (char *)malloc(baselen + extlen + 2);
		strncpy(outfile, argv[i], baselen);
		*(outfile + baselen) = '.';
		strcpy(outfile + baselen + 1, extension);
	}

	fin = freopen(asmfile, "r", stdin);
	if (fin == NULL) {
		if (!has_dot_asm) {
			strcpy(asmfile + baselen, ".asm");
			fin = freopen(asmfile, "r", stdin);
		}
		if (fin == NULL) {
			fprintf(stderr,"Cannot open input file: %s\n",
				asmfile);
			exit(1);
		}
	}

	if (dashl) {
		listing = fopen(lstfile,"w");
		if( listing == NULL ) {
			fprintf(stderr,"Cannot open file: %s for writting.\n",
				lstfile);
			exit(1);
		}
	}

	emitopen(outfile, dashF, dashA);

	syminit();
	fatal = abort_asap = 0;
	lineno = 1;
	pass=0;
	lc = 0x2000;

	/*
	** P A S S    1
	*/
	if (!use_stdout) printf("Begin Pass #1\n");
	yyparse();
	if (fatal) quit(1);

	rewind(fin);
	lineno = 1;
	pass++;
	lc = 0x2000;
	emitaddr(lc);

	/*
	** P A S S    2
	*/
        if (!use_stdout) printf("Begin Pass #2\n");
	yyparse();
	if (fatal) quit(1);

	emitclose();
	fclose(fin);
	if( dashl )
		fclose(listing);
	exit(0);

}


void quit(int exit_code)
{
	fclose(fin);
	emitclose();
	exit(exit_code);
}


void error(const char *fmt, ...)
{
	va_list args;

	abort_asap++;
        fatal++;
	va_start(args, fmt);
        fprintf(stderr,"Error, line %d, ",lineno);
	vfprintf(stdout, fmt, args);
	fprintf(stdout, ".\n");
	quit(1);
}


void warn(const char *fmt, ...)
{
	va_list args;

        fatal++;
	va_start(args, fmt);
        fprintf(stderr,"Warning, line %d, ",lineno);
	vfprintf(stdout, fmt, args);
	fprintf(stdout, ".\n");
}


