/* ----------------------------------------------------------------------
 * FILE: main.c
 * PACKAGE: as31 - 8031/8051 Assembler.
 *
 * DESCRIPTION:
 *	The file contains main(). It handles the arguments and makes
 *	sure that pass 1 is done before pass 2 etc...
 *
 * REVISION HISTORY:
 *	Jan. 19, 1990 - Created. (Ken Stauffer)
 *
 * AUTHOR:
 *	All code in this file written by Ken Stauffer (University of Calgary).
 *	January, 1990. "Written by: Ken Stauffer"
 *
 *	April, 2000, Paul Stoffregen: see Makefile for details
 */

#include <stdio.h>
#include <setjmp.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

#include "as31.h"


void do_assemble(GtkWidget *widget, gpointer *data);
gint delete_handler(GtkWidget *widget, gpointer *data);
void print(const char *fmt, ...);
void vprint(const char *fmt, va_list args);


/* global variables */
unsigned long lc;
char *asmfile;
int fatal=0, abort_asap=0;
int pass=0;
int dashl=0;
FILE *listing=NULL, *fin=NULL;

GtkWidget *file_label, *file_entry, *otype_combo;
GtkWidget *list_check, *go_button;
GtkWidget *otype_label, *output_text;
GtkWidget *main_window, *main_vbox;
GtkWidget *top_hbox, *middle_hbox, *bottom_hbox;


int main(int argc, char **argv)
{
	GList *otypelist;
	const char *desc;
	int n;

	gtk_init(&argc, &argv);

	/* create all the gtk widgets... it's a pain to type in all */
	/* this stuff, maybe someday I'll have to learn about glade */

	file_label = gtk_label_new("ASM File: ");
	gtk_label_set_justify(GTK_LABEL(file_label), GTK_JUSTIFY_RIGHT);
	gtk_widget_show(file_label);

	otype_label = gtk_label_new("Output Format");
	gtk_label_set_justify(GTK_LABEL(otype_label), GTK_JUSTIFY_RIGHT);
	gtk_widget_show(otype_label);

	file_entry = gtk_entry_new();
	/* fill in a default filename, from a previous session ??? */
	gtk_widget_show(file_entry);

	list_check = gtk_check_button_new_with_label("Create List File");
	/* check or unchecked based on previous usage */
	gtk_widget_show(list_check);

	otype_combo = gtk_combo_new();
	otypelist = NULL;
	n = 0;
	while ((desc = emit_desc_lookup(n++)) != NULL) {
		otypelist = g_list_append(otypelist, strdup(desc));
	}
	gtk_combo_set_popdown_strings(GTK_COMBO(otype_combo), otypelist);
	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(otype_combo)->entry),
		FALSE);
	/* set to same output format as a previous run ?? */
	gtk_widget_show(otype_combo);

	go_button = gtk_button_new_with_label("Assemble");
	gtk_widget_show(go_button);

	output_text = gtk_text_new(NULL, NULL);
	gtk_text_set_editable(GTK_TEXT(output_text), FALSE);
	gtk_widget_show(output_text);

	top_hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(top_hbox), file_label, FALSE, FALSE, 2);
	gtk_box_pack_start(GTK_BOX(top_hbox), file_entry, TRUE, TRUE, 2);
        gtk_widget_show(top_hbox);

	middle_hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(middle_hbox), otype_label, FALSE, FALSE, 2);
	gtk_box_pack_start(GTK_BOX(middle_hbox), otype_combo, FALSE, FALSE, 2);
        gtk_widget_show(middle_hbox);

	bottom_hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(bottom_hbox), go_button, FALSE, FALSE, 2);
	gtk_box_pack_start(GTK_BOX(bottom_hbox), list_check, FALSE, FALSE, 2);
        gtk_widget_show(bottom_hbox);

	main_vbox = gtk_vbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(main_vbox), top_hbox, FALSE, FALSE, 2);
	gtk_box_pack_start(GTK_BOX(main_vbox), middle_hbox, FALSE, FALSE, 2);
	gtk_box_pack_start(GTK_BOX(main_vbox), bottom_hbox, FALSE, FALSE, 2);
	gtk_box_pack_start(GTK_BOX(main_vbox), output_text, TRUE, TRUE, 2);
        gtk_widget_show(main_vbox);

        main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
        gtk_container_add(GTK_CONTAINER(main_window), main_vbox);
        gtk_widget_show(main_window);

	/* attach signal handlers so we can actually do something */

	gtk_signal_connect(GTK_OBJECT(main_window), "delete_event",
		GTK_SIGNAL_FUNC(delete_handler), NULL);

	gtk_signal_connect(GTK_OBJECT(go_button), "pressed",
		GTK_SIGNAL_FUNC(do_assemble), NULL);

	/* that's it, everything past here is a callback from GTK */

	print("WARNING: this is an alpha-quality release.\n");
	print("It WILL crash the second time you assemble !!\n");
	print("Please report any problems that occur on the\n");
	print("first assembly to: paul@pjrc.com\n");

	gtk_main();
	return 0;
}


/* actually do the assembly.  The entire assembly is done from */
/* start to finish in this callback attached to the "assemble now" */
/* button.  It's not so nice to spend a lot of CPU time in a */
/* callback, because all the gtk widgets stop responding until */
/* we're done, but it keeps things simple, and at least the */
/* window manager takes care of raise/lower and reposition, */
/* ...unlike ms windoze where those are handled by the */
/* application's event loop.  It would be "nice" to launch a */
/* thread that would do the assembly, but for a simple free */
/* software project (where the assembly completes rapidly on */
/* any modern cpu), I don't want to bother with all the unpleasant */
/* threading issues */

void
do_assemble(GtkWidget *widget, gpointer *data)
{
	char *outfile=NULL, *lstfile=NULL;
	const char *dashF=NULL, *filename, *extension;
	int has_dot_asm=0;
	int len, baselen, extlen;


	/* collect info from the GTK widgets */

	dashl = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(list_check));

	filename = gtk_entry_get_text(GTK_ENTRY(file_entry));
	if (filename == NULL || *filename == '\0') {
		print("You need to specify the ASM file\n");
		return;
	}

	dashF = emit_desc_to_name_lookup(gtk_entry_get_text(
		GTK_ENTRY(GTK_COMBO(otype_combo)->entry)));
	if (dashF == NULL) {
		print("Please select a valid output type\n");
		return;
	}


	/* now we can begin working like normal */

	extension = emit_extension(dashF);
	extlen = strlen(extension);

	len = baselen = strlen(filename);
	if (len >= 4 && strcasecmp(filename + len - 4, ".asm") == 0) {
		has_dot_asm = 1;
		baselen -= 4;
	}
	
	asmfile = (char *)malloc(baselen + 5);
	strcpy(asmfile, filename);

	if (dashl) {
		lstfile = (char *)malloc(baselen + 5);
		strncpy(lstfile, filename, baselen);
		strcpy(lstfile + baselen, ".lst");
	}

	outfile = (char *)malloc(baselen + extlen + 2);
	strncpy(outfile, filename, baselen);
	*(outfile + baselen) = '.';
	strcpy(outfile + baselen + 1, extension);

	fin = freopen(asmfile, "r", stdin);
	if (fin == NULL) {
		if (!has_dot_asm) {
			strcpy(asmfile + baselen, ".asm");
			fin = freopen(asmfile, "r", stdin);
		}
		if (fin == NULL) {
			fprintf(stderr,"Cannot open input file: %s\n",
				asmfile);
			return;
		}
	}

	if (dashl) {
		listing = fopen(lstfile,"w");
		if (listing == NULL) {
			print("Cannot open file: %s for writting.\n",
				lstfile);
			return;
		}
	}

	emitopen(outfile, dashF, NULL);

	syminit();
	fatal = 0;
	lineno = 1;
	pass=0;
	lc = 0x2000;

	/*
	** P A S S    1
	*/
	print("Begin Pass #1\n");
	yyparse();
	if (fatal) {
		print("Errors in pass1, assembly aborted");
		return;
	}

	rewind(fin);
	lineno = 1;
	pass++;
	lc = 0x2000;
	emitaddr(lc);

	/*
	** P A S S    2
	*/
        print("Begin Pass #2\n");
	yyparse();
	emitclose();
	fclose(fin);
	if (dashl) fclose(listing);
	if (fatal) {
		print("Errors in pass2, output file may contain errors");
	} else {
		print("Assembly complete, no errors :)");
	}
	return;

}

/* quit the program if they close it */

gint
delete_handler(GtkWidget *widget, gpointer *data)
{
	gtk_main_quit();
	return FALSE;
}

/* the parser, lexer and other stuff that actually do the */
/* assembly will call to these two functions to report any */
/* errors or warning.  error() calls exit() in the command */
/* line version, but the abort_asap flag was added, and the */
/* parser check it at the end of every line */

void error(const char *fmt, ...)
{
	va_list args;

	abort_asap++;
	fatal++;
	va_start(args, fmt);
	vprint(fmt, args);

        fprintf(stderr,"Error, line %d, ",lineno);
	vfprintf(stdout, fmt, args);
	fprintf(stdout, ".\n");
}

void warn(const char *fmt, ...)
{
	va_list args;

        fatal++;
	va_start(args, fmt);
	vprint(fmt, args);

        print("Warning, line %d, ",lineno);
	vprint(fmt, args);
	print(".\n");
}

/* these two functions provide printf (without the 'f') */
/* that writes to the display window in the main dialog */
/* box, instead of to stdout or a stream. */

void print(const char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	vprint(fmt, args);
}

void vprint(const char *fmt, va_list args)
{
	char buf[1024];

	vsnprintf(buf, sizeof(buf), fmt, args);
	printf("%s", buf);
	gtk_text_insert(GTK_TEXT(output_text), NULL, NULL,
		NULL, buf, strlen(buf));
}

